@@ -3,7 +3,20 @@ import cqlat_utils as cu
 from qlat_utils.cache import *
 from qlat_utils.rng_state import *
 
+import math
 import sys
+import os
+
+def getenv(*names, default = None):
+    assert len(names) > 0
+    for name in names:
+        val = os.getenv(name)
+        if val is not None:
+            displayln_info(0, f"{name}='{val}'")
+            return val
+    val = default
+    displayln_info(0, f"{names[0]}='{val}' (default)")
+    return val
 
 def get_arg(option, default = None):
     argv = sys.argv
@@ -24,25 +37,20 @@ def random_permute(l, rs):
     return cu.random_permute(l, rs)
 
 def show_memory_usage():
-    import psutil
-    rss = psutil.Process().memory_info().rss / (1024 * 1024 * 1024)
-    displayln_info(f"show_memory_usage: rss = {rss:.6f} GB")
-    malloc_stats_info()
+    try:
+        import psutil
+        rss = psutil.Process().memory_info().rss / (1024 * 1024 * 1024)
+        displayln_info(f"show_memory_usage: rss = {rss:.6f} GB")
+        # displayln_info_malloc_stats()
+    except:
+        displayln_info(f"show_memory_usage: no psutil.")
 
-def get_all_caches_info():
-    return cu.get_all_caches_info()
+def displayln_malloc_stats():
+    return cu.displayln_malloc_stats()
 
-def clear_all_caches():
-    # clean python level cache and then C++ level cache
-    clean_cache()
-    cu.clear_all_caches()
-
-def malloc_stats():
-    return cu.malloc_stats()
-
-def malloc_stats_info():
+def displayln_info_malloc_stats():
     if get_id_node() == 0:
-        return malloc_stats()
+        return displayln_malloc_stats()
 
 def lazy_call(f, *args, **kwargs):
     is_thunk = True
@@ -88,3 +96,21 @@ def rel_mod_sym(x, size):
     else:
         assert 2 * x == size
         return 0
+
+def c_sqr(x):
+    return sum([ sqr(v) for v in x ])
+
+def c_rel_mod(x, size):
+    l = len(size)
+    assert l == len(x)
+    return [ rel_mod(x[i], size[i]) for i in range(l) ]
+
+def c_rel_mod_sqr(x, size):
+    l = len(size)
+    assert l == len(x)
+    return sum([ sqr(rel_mod(x[i], size[i])) for i in range(l) ])
+
+def phat_sqr(q, size):
+    l = len(size)
+    assert l == len(q)
+    return 4 * sum([ sqr(math.sin(math.pi * (q[i] % size[i]) / size[i])) for i in range(l) ])
